unit hmgraphe;

interface

uses windows, SysUtils, Graphics, Classes, Printers,
   Forms, Controls, Buttons, StdCtrls, ExtCtrls, Dialogs, inifiles,
   jmmUtil, math;

const
   MaxVecteurMult    = 256;
   MaxVecteurOsc = 32768;
   CouleurAxe: Tcolor = clGray;
   CouleurVoieOsc: array[1..2] of Tcolor = (clBlue,clRed);
   NbreVoiesMax  = 4;
   voieT         = NbreVoiesMax + 1;
   voieX         = 0;
   voieF         = 0;
   voieA         = 1;
   voieGain      = 1;
   voiePhase     = 2;
   extensionHP   = '.HP';

type
   TCalculIncertitude = (ciNone, ciFreq, ciTemps, ciTension, ciDuty,
      ciPhase, ciTensionEff);
   indiceVecteurMult  = 0..MaxVecteurMult;
   indiceVecteurOsc  = 0..MaxVecteurOsc;
   vecteurMult        = array[indiceVecteurMult] of double;
   vecteurOscInt      = array[indiceVecteurOsc] of integer;
   indiceVoieMult = voieX..VoieT;
   indiceVoieBode = voieF..VoiePhase;

   TvoieRecord = record
      mini, maxi:        double;
      Nom: string;
      Command, CommandMX: ansiString;
      Unite, Signif: string;
      Active, Positive:  boolean;
      CalculIncertitude: TCalculIncertitude;
      { mode multimtre }
      NumVoie:           byte;
   end;

   TvoieHP = class
      mini, maxi:        double;
      Nom : string;
      Command, CommandMX: ansiString;
      Unite, Signif:     string;
      Commentaire:       string;
      Active, Positive:  boolean;
      CalculIncertitude: TCalculIncertitude;
      erreurLecture:     boolean;
      NumVoie:           byte;
      constructor Create;
      procedure load(ini: TmemIniFile; nomIndex: String);
      destructor Destroy; override;
   end;

   Tvoie = class(TvoieHP)
      a, b:        double; { entier = rel*a+b }
      valeurCourante, oldValeur: double;
      incertitudeCourante: double;
      valeur:      vecteurMult;
      incertitude: vecteurMult; { mode multimtre }
      AxeLog:      boolean;
      LabelVoie:   Tlabel;
      GBvoie:      TgroupBox;
      AxedB:       boolean;
      posCurseur:  integer;
      constructor Create;
      destructor Destroy; override;
      function GrandeurToEcran(x: double): integer;
      procedure SetPhase;
      procedure SetIncertitude;
      function StrValeur(nombre: double): string;
      function StrValeurFixed(nombre: double): string;
      function AffVoltmetre: String;
   end;

   Tgraphe = class
      dimPoint:       integer;
      Canvas:         Tcanvas;
      Quadrillage:    boolean;
      ErreurLectureGlb: boolean;
      voie:           array[indiceVoieMult] of Tvoie;
      NbrePoints:     integer;
      DerniereVoie:   IndiceVoieMult;
      LimiteCourbe:   Trect;
      PanelErreurLect: Tlabel;
      avecAxeY:       boolean;
      acqAutoEnCours: boolean;
      constructor Create;
      procedure DrawG(Arect: Trect);
      procedure EnregistrePointCourant;
      procedure AffichePointCourant;
      procedure AfficheCurseur;
      procedure TraceCroix(xr, yr, dxr, dyr: double; v: indiceVoieMult);
      procedure DrawAxis;
      procedure SaveData(const NomFichier: String);
      destructor Destroy; override;
      procedure LineG(x1, y1, x2, y2: integer);
      procedure tri;
   end;

   TgrapheOsc = class
      Canvas:         Tcanvas;
      Quadrillage:    boolean;
      voie:  array[1..2] of vecteurOscInt;
      NbrePoints:     integer;
      Ax,Ay,Bx,By : double;
      MaxY,MinY : integer;
      posX,posY : array[1..2,1..2] of integer;
      constructor Create;
      procedure DrawG(Arect: Trect);
      procedure GrandeurToEcran(x,y : integer;var xe,ye : integer);
      procedure LineG(x1, y1, x2, y2: integer);
   end;


const
   CouleurVoie: array[indiceVoieMult] of Tcolor =
      (clBlack, clRed, clGreen, clBlue, clPurple, clBlack);
   VoieDefaut: array[indiceVoieMult] of TvoieRecord =
      ((mini: 100; maxi: 10000; Nom: 'f'; command: 'FREQ'; commandMX: 'FREQ';
      unite: 'Hz'; Signif: 'frquence'; active: True; positive: True;
      calculIncertitude: ciFreq; Numvoie: 1),
      (mini: 0; maxi: 5; Nom: 'Ve'; command: 'VRMS'; commandMX: 'AC';
      unite: 'V'; signif: 'efficace'; active: True; positive: True;
      calculIncertitude: ciTensionEff; Numvoie: 1),
      (mini: 0; maxi: 5; Nom: 'Vs'; command: 'VRMS'; commandMX: 'AC';
      unite: 'V'; signif: 'efficace'; active: True; positive: True;
      calculIncertitude: ciTensionEff; Numvoie: 2),
      (mini: -180; maxi: +180; Nom: ''; command: 'PHAS'; commandMX: 'PHAS';
      unite: ''; signif: 'phase(1,2)'; active: True; positive: False;
      calculIncertitude: ciPhase; Numvoie: 1),
      (mini: 0; maxi: 5; Nom: 'V'; command: 'VAMP'; commandMX: 'AMPL';
      unite: 'V'; signif: 'amplitude'; active: False; positive: True;
      calculIncertitude: ciTension; Numvoie: 2),
      (mini: 100; maxi: 10000; Nom: 'f'; command: 'FREQ'; commandMX: 'FREQ';
      unite: 'Hz'; Signif: 'frquence'; active: True; positive: True;
      calculIncertitude: ciFreq; Numvoie: 1));

   NsauveCommande = 80;

   SauveCommande: array[0..NsauveCommande] of String =
      ('[FREQuency]',
      'Signif=frquence',
      'Nom=f',
      'Command=FREQ',
      'Unite=Hz',
      'Positive=1',
      'Incertitude=1',
      'Maxi=1e6',
      'Mini=1e2',
      '[PERiod]',
      'Signif=priode',
      'Nom=T',
      'Command=PER',
      'Unite=s',
      'Positive=1',
      'Incertitude=2',
      'Maxi=1',
      'Mini=0',
      '[PHASe]',
      'Signif=phase(1,2)',
      'Nom=',
      'Command=PHAS',
      'Unite=',
      'Positive=0',
      'Incertitude=5',
      'Maxi=180',
      'Mini=-180',
      '[RISetime]',
      'Signif=temps de monte',
      'Nom=tm',
      'Command=RIS',
      'Unite=s',
      'Positive=1',
      'Incertitude=2',
      'Maxi=1',
      'Mini=0',
      '[DUTYcycle]',
      'Command=DUTY',
      'Nom=rc',
      'Signif=rapport cyclique',
      'Unite=',
      'Positive=1',
      'Incertitude=4',
      'Maxi=1',
      'Mini=0',
      '[VAVerage]',
      'Command=VAV',
      'Nom=Vm',
      'Signif=moyenne',
      'Unite=V',
      'Positive=0',
      'Incertitude=6',
      'Maxi=5',
      'Mini=0',
      '[VPP]',
      'Command=VPP',
      'Nom=Vcc',
      'Signif=crte-crte',
      'Unite=V',
      'Positive=1',
      'Incertitude=3',
      'Maxi=5',
      'Mini=0',
      '[VRMS]',
      'Command=VRMS',
      'Nom=Veff',
      'Signif=efficace',
      'Unite=V',
      'Positive=1',
      'Incertitude=6',
      'Maxi=5',
      'Mini=0',
      '[VAMPlitude]',
      'Command=VAMP',
      'Nom=Vamp',
      'Signif=amplitude',
      'Unite=V',
      'Positive=1',
      'Incertitude=3',
      'Maxi=5',
      'Mini=0');

var
   BaseDeTemps:       double;
   FrequenceCourante: double;
   FullScale:         array[1..4] of double;

implementation

constructor TvoieHP.Create;
begin
   inherited Create;
   nom  := '';
   Signif := '';
   Unite := '';
   Commentaire := '';
   Command := '';
   commandMX := '';
   Maxi := 1;
   Mini := 0;
   Active := False;
   Positive := True;
   CalculIncertitude := ciNone;
   NumVoie := 1;
end; // TvoieHP.Create

constructor Tvoie.Create;
begin
   inherited Create;
   A := 1;
   B := 0;
   ValeurCourante := Nan;
   oldValeur := Nan;
   incertitudeCourante := 0;
   calculIncertitude := ciNone;
   NumVoie := 1;
   AxeLog  := False;
end;{--- Tvoie.Create ---}

destructor Tvoie.Destroy;
begin
   inherited Destroy;
end;

destructor TvoieHP.Destroy;
begin
   inherited Destroy;
end;

function Tvoie.GrandeurToEcran(x: double): integer;
begin
   if isNan(x) then
      Result := -1
   else begin
      if axeLog then
         x := log10(x);
      GrandeurToEcran := round(A * x + B);
   end;
end;

procedure TgrapheOsc.GrandeurToEcran(x,y: integer;var xe,ye : integer);
begin
      xe := round(Ax * x + Bx);
      ye := round(Ay * y + By);
end;

procedure Tgraphe.Tri;
 { Tri Shell Meyer-Baudoin p 456 }
 { origine vecteur en 0 }
var
   cle: double;
   i, j: integer;
   v: indiceVoieMult;
   SauvePoint: array[indiceVoieMult] of double;
const
   deuxPi = 360.0;
begin
   i := 1;
   while (i < NbrePoints) do begin
      for v := voieX to NbreVoiesMax do
         sauvePoint[v] := voie[v].valeur[i];
      cle := voie[voieX].valeur[i];
      j := pred(i);
      while (j >= 0) and
            (voie[voieX].valeur[j] > cle) do begin
         for v := voieX to NbreVoiesMax do
            voie[v].valeur[succ(j)] := voie[v].valeur[j];
         Dec(j);
      end;
      for v := voieX to NbreVoiesMax do
         voie[v].valeur[succ(j)] := sauvePoint[v];
      Inc(i);
   end;
   for v := voieA to NbreVoiesMax do
      with voie[v] do
         if active and (unite = '') then
            for i := 1 to pred(NbrePoints) do
               Valeur[i] := Valeur[i] - round(
                  (Valeur[i] - Valeur[pred(i)]) / DeuxPi) * DeuxPi;
end;{tri}

procedure Tgraphe.EnregistrePointCourant;
var
   i: indiceVoieMult;
begin
   if ErreurLectureGlb then begin
      PanelErreurLect.Caption := 'Donnes incorrectes';
      PanelErreurLect.Visible := True;
      exit;
   end;
   if not AcqAutoEnCours then
      for i := voieX to DerniereVoie do
         with voie[i] do
            if abs(valeurCourante - oldValeur) > (maxi - mini) / 32 then begin
               PanelErreurLect.Caption := 'Donnes instables (cf Avg)';
               PanelErreurLect.Visible := True;
               exit;
            end;
   for i := voieX to DerniereVoie do
      with voie[i] do begin
         if isNan(valeurCourante) then
            exit;
         valeur[NbrePoints] := valeurCourante;
         incertitude[NbrePoints] := incertitudeCourante;
      end;
   Voie[voieT].valeur[NbrePoints] := Voie[voieT].valeurCourante;
   Inc(NbrePoints);
   AffichePointCourant;
end;

procedure Tgraphe.TraceCroix(xr, yr, dxr, dyr: double; v: indiceVoieMult);
var
   x, y, dx, dy, x1, x2, y1, y2: integer;
begin
   dxr := dxr * 2.58; // 99 %
   dyr := dyr * 2.58; // 99 %        
   x1  := voie[voieX].GrandeurToEcran(xr - dxr);
   x2  := voie[voieX].GrandeurToEcran(xr + dxr);
   dx  := (x2 - x1) div 2;
   if dx < 1 then dx := 1;
   x := (x1 + x2) div 2;
   y1 := voie[v].grandeurToEcran(yr - dyr);
   y2 := voie[v].grandeurToEcran(yr + dyr);
   dy := (y1 - y2) div 2;
   if dy < dimPoint then
      dy := dimPoint;
   y := (y1 + y2) div 2;
   Canvas.Pen.color := couleurVoie[v];
   Canvas.Brush.color := couleurVoie[v];
   Canvas.Ellipse(x-dx,y-dy,x+dx,y+dy);
{
   Canvas.MoveTo(x, y - dy);
   Canvas.LineTo(x, y + dy);
   Canvas.MoveTo(x - dx, y);
   Canvas.LineTo(x + dx, y);
}
end;

procedure Tgraphe.AffichePointCourant;
var
   i: indiceVoieMult;
begin
   if ErreurLectureGlb then
      exit;
   canvas.Pen.Mode  := pmNotXor;
   canvas.pen.Width := 1;
   canvas.brush.style := bsSolid;
   for i := voieA to DerniereVoie do
      TraceCroix(voie[voieX].valeurCourante,
         voie[i].valeurCourante,
         voie[voieX].incertitudeCourante,
         voie[i].incertitudeCourante, i);
end;

procedure Tgraphe.AfficheCurseur;

   procedure TraceLoc(v: indiceVoieMult);
   var
      x, y: integer;
   begin
      x := voie[voieX].posCurseur;
      y := voie[v].posCurseur;
      if y < 0 then
         exit;
      Canvas.Pen.color := couleurVoie[v];
      Canvas.Brush.color := couleurVoie[v];
      Canvas.MoveTo(x - 3 * dimPoint, y);
      Canvas.LineTo(x + 3 * dimPoint, y);
      Canvas.MoveTo(x, y - 3 * dimPoint);
      Canvas.LineTo(x, y + 3 * dimPoint);
   end;

var
   v: indiceVoieMult;
begin
   if ErreurLectureGlb then
      exit;
   canvas.Pen.Mode  := pmNotXor;
   canvas.pen.Width := 3;
   canvas.brush.style := bsSolid;
   if voie[voieX].posCurseur > 0 then // efface l'ancien
      for v := voieA to DerniereVoie do
         TraceLoc(v);
   with voie[voieX] do
      posCurseur := GrandeurToEcran(valeurCourante);
   for v := voieA to DerniereVoie do begin
      with voie[v] do
         PosCurseur := grandeurToEcran(valeurCourante);
      TraceLoc(v);
   end;
end;

procedure Tgraphe.DrawG(Arect: Trect);
var
   j: indiceVoieMult;
   i: integer;
begin
   LimiteCourbe := Arect;
   canvas.font.Name := 'Regressi';
   canvas.font.size := 10;
   canvas.Pen.Mode := pmCopy;
   Dec(LimiteCourbe.bottom, 2*canvas.font.Height);
   Inc(LimiteCourbe.top, 2);
   canvas.brush.color := clWindow;
   canvas.font.color  := couleurAxe;
   canvas.FillRect(canvas.clipRect);
   canvas.brush.style := bsSolid;
   canvas.pen.Width := 1;
   canvas.pen.color := clGray;
   with voie[voieX], LimiteCourbe do begin
      dimPoint := succ((bottom - top) div 220);
      if axeLog then begin
         A := (right - left) / log10(Maxi / Mini);
         B := left - log10(Mini) * voie[voieX].A;
      end
      else begin
         A := (right - left) / (Maxi - Mini);
         B := left - Mini * voie[voieX].A;
      end;
   end;
   if quadrillage then
      drawAxis;
   canvas.Pen.style := psSolid;
   for j := voieA to DerniereVoie do
      with LimiteCourbe, voie[j] do begin
         A := (top - bottom) / (Maxi - Mini);
         B := bottom - Mini * A;
      end;
   canvas.pen.Width := 1;
   for i := 0 to pred(NbrePoints) do
      for j := voieA to DerniereVoie do
         TraceCroix(voie[voieX].valeur[i],
            voie[j].valeur[i],
            voie[voieX].Incertitude[i],
            voie[j].Incertitude[i], j);
   for j := voieX to DerniereVoie do
      voie[j].posCurseur := -1;
end;

procedure TgrapheOsc.DrawG(Arect: Trect);

procedure DrawAxis;
var i, x, y : integer;
begin
   with canvas, Arect do begin
      pen.color := CouleurAxe;
      pen.Style := psDot;
      for i := 0 to 8 do begin
         y := round(top + i * (bottom - top) div 8);
         LineG(left, y, right, y);
      end;
      for i := 0 to 10 do begin
          x := round(left + i * (right - left) div 10);
          LineG(x, top, x, bottom);
      end;
      pen.Style := psSolid;
   end;
end;{ DrawAxis }

var
   j: integer;
   i: integer;
   xe,ye : integer;
begin
   canvas.Pen.Mode := pmCopy;
   canvas.brush.color := clWindow;
   canvas.font.color  := couleurAxe;
   canvas.FillRect(canvas.clipRect);
   canvas.brush.style := bsSolid;
   canvas.pen.Width := 1;
   canvas.pen.color := clGray;
   with Arect do begin
       AX := (right - left) / NbrePoints;
       BX := left;
   end;
   if quadrillage then drawAxis;
   canvas.Pen.style := psSolid;
   for j := 1 to 2 do
      with Arect do begin
         AY := (top - bottom) / (MaxY - MinY);
         BY := bottom - MinY * AY;
      end;
   canvas.pen.Width := 1;
   for j := 1 to 2 do begin
       canvas.pen.color := couleurVoieOsc[j];
       grandeurToEcran(0,voie[j,0],xe,ye);
       canvas.moveTo(xe,ye);
       for i := 1 to pred(NbrePoints) do begin
           grandeurToEcran(i,voie[j,i],xe,ye);
           canvas.lineTo(xe,ye);
       end;
   end;
   canvas.Pen.style := psDot;
   for j := 1 to 2 do begin
      canvas.pen.color := couleurVoieOsc[j];
      for i := 1 to 2 do begin
      if posX[j,i]<>0 then begin
         grandeurToEcran(posX[j,i],0,xe,ye);
         lineG(xe,arect.top,xe,arect.bottom);
      end;
      if posY[j,i]<>0 then begin
         grandeurToEcran(0,posY[j,i],xe,ye);
         lineG(arect.left,ye,arect.right,ye);
      end;
      end;
   end; // j
end;

destructor Tgraphe.Destroy;
var
   j: indiceVoieMult;
begin
   for j := voieX to voieT do
      voie[j].Free;
end;

constructor Tgraphe.Create;
var
   j: indiceVoieMult;
begin
   inherited Create;
   for j := voieX to voieT do
      voie[j] := Tvoie.Create;
   voie[voieX].axeLog := True;
   quadrillage := True;
   erreurLectureGlb := False;
   DerniereVoie := voieA;
   NbrePoints := 0;
end;

constructor TgrapheOsc.Create;
begin
   inherited Create;
   quadrillage := True;
   NbrePoints := 0;
end;

procedure Tgraphe.LineG(x1, y1, x2, y2: integer);
begin
   canvas.moveTo(x1, y1);
   canvas.lineTo(x2, y2);
end;

procedure TgrapheOsc.LineG(x1, y1, x2, y2: integer);
begin
   canvas.moveTo(x1, y1);
   canvas.lineTo(x2, y2);
end;

procedure Tgraphe.DrawAxis;

   procedure traceAxeLogx;
   var
      i, j: integer;
      xs: integer;
      intermediaire: boolean;
   begin
      with canvas, limiteCourbe, voie[voieX] do begin
         Intermediaire := log10(Maxi / Mini) <= 4;
         for i := round(log10(Mini)) to round(log10(Maxi)) do begin
            xs := grandeurToEcran(power(10,i));
            pen.style := psSolid;
            LineG(xs, top, xs, bottom);
            TextOut(xs, bottom + 2, formatCourt(power(10,i), Unite));
            pen.style := psDot;
            if intermediaire then
               for j := 2 to 9 do begin
                  xs := grandeurToEcran(power(10,i + log10(j)));
                  LineG(xs, top, xs, bottom);
               end;{for j}
         end; {for i}
      end;
   end;{ TraceAxeLogx }

   procedure traceAxeX;
   var
      i:  integer;
      xs: integer;
      x:  double;
   begin
      with canvas, limiteCourbe, voie[voieX] do begin
         for i := 1 to 4 do begin
            x  := i * Maxi / 5;
            xs := grandeurToEcran(x);
            LineG(xs, top, xs, bottom);
            TextOut(xs, bottom + 2, formatCourt(x, unite));
         end; {for i}
      end;
   end;{ TraceAxeX }

var
   i, y: integer;
begin
   with canvas, LimiteCourbe do begin
      pen.color := CouleurAxe;
      pen.Style := psDot;
      for i := 0 to 10 do begin
         y := round(top + i * (bottom - top) div 10);
         LineG(left, y, right, y);
      end;
      if voie[voieX].axeLog then
         traceAxeLogX
      else
         traceAxeX;
      TextOut(right + 2 * font.Height, bottom + 2, voie[voieX].nom);
      pen.Style := psSolid;
   end;
end;{ DrawAxis }

procedure Tgraphe.SaveData(const NomFichier: String);
var
   fichier: textFile;
   formatRW3: boolean;

   procedure EcritPage;
   var
      j: integer;
      i: indiceVoieMult;
   begin
      if formatRW3 then begin
         writeln(fichier, SymbReg, '1 PAGE COMMENT');
         writeln(fichier, '');
         writeln(fichier, SymbReg2, NbrePoints, ' VALEUR VAR');
         for j := 0 to pred(NbrePoints) do begin
            Write(fichier, FloatToStrPoint(voie[voieX].valeur[j]), ' ');
            for i := succ(voieX) to DerniereVoie do
               Write(fichier, FloatToStrPoint(voie[i].valeur[j]), ' ');
            writeln(fichier);
         end;
      end
      else begin
         for j := 0 to pred(NbrePoints) do begin
            for i := voieX to DerniereVoie do
               Write(fichier, FloatToStr(voie[i].valeur[j]), crTab);
            writeln(fichier);
         end;
      end;
   end; {ecritPage}

   procedure ecritEnTete;
   var
      j: indiceVoieMult;
      N: integer;
   begin
      if formatRW3 then begin
         N := DerniereVoie + 1;
         writeln(fichier, 'EVARISTE REGRESSI WINDOWS 1.0');
         writeln(fichier, symbReg, N, ' NOM VAR');
         for j := voieX to DerniereVoie do
            writeln(fichier, voie[j].nom);
         writeln(fichier, symbReg, N, ' UNITE VAR');
         for j := voieX to DerniereVoie do
            writeln(fichier, voie[j].unite);
         writeln(fichier, symbReg, N, ' SIGNIF VAR');
         for j := voieX to DerniereVoie do
            writeln(fichier, voie[j].commentaire);
         writeln(fichier, symbReg, '1 TRIGO');
         writeln(fichier, Ord(True));
         writeln(fichier, symbReg, '1 ACQUISITION');
         writeln(fichier, Application.ExeName);
      end
      else begin
         writeln(fichier, 'HP54600');
         for j := voieX to DerniereVoie do
            Write(fichier, voie[j].nom, crTab);
         writeln(fichier);
         for j := voieX to DerniereVoie do
            Write(fichier, voie[j].unite, crTab);
         writeln(fichier);
         for j := voieX to DerniereVoie do
            Write(fichier, voie[j].signif, crTab);
         writeln(fichier);
      end;
   end; { ecritEnTete }

begin { saveData }
   FormatRW3 := AnsiUpperCase(ExtractFileExt(NomFichier)) = '.RW3';
   AssignFile(fichier, NomFichier);
   Rewrite(fichier);
   ecritEnTete;
   tri;
   ecritPage;
   CloseFile(fichier);
end;

procedure Tvoie.SetPhase;
begin
   if not isNan(valeurCourante) and
      (command = 'PHAS') then begin
      { oscillo renvoie entre -360 et +360 ramen entre -180 et +180 }
      if valeurCourante < -180 then
         valeurCourante := valeurCourante + 360;
      if valeurCourante > 180 then
         valeurCourante := valeurCourante - 360;
   end;
end;

procedure Tvoie.SetIncertitude;  // en cart-type ? donc LSB/rac(12)
var
   deltaT, deltaV: double;
begin
   incertitudeCourante := 0;
   deltaT := baseDeTemps / 2048; // 1/2 LSB sur 1024 points
   deltaV := FullScale[NumVoie] / 512; // 1/2 LSB sur 8 bits
   if not isNan(valeurCourante) then begin
      case calculIncertitude of
         ciNone: ;
         ciFreq: incertitudeCourante  := valeurCourante / 1e4 + sqr(valeurCourante) * DeltaT;
         // 0,01% lecture + 0,1% screenWidth
         ciTemps: incertitudeCourante := valeurCourante / 1e4 + DeltaT;
         ciTension: incertitudeCourante := DeltaV;
         // calibre : 2% pleine chelle
         // offset : 0,5% offset + 1% pleine chelle
         ciDuty: incertitudeCourante  := (1 + valeurCourante) * DeltaT * FrequenceCourante;
         ciPhase: incertitudeCourante :=
               (360 + valeurCourante) * DeltaT * FrequenceCourante;
         ciTensionEff: incertitudeCourante := DeltaV / 2; // effet de la moyenne
      end;
   end;
end;

function Tvoie.StrValeurFixed(nombre: double): string;
var
   digits: integer;
begin
   digits := trunc(-log10(abs(nombre * 100)));
   if digits < 0 then
      Result := IntToStr(round(nombre))
   else
      Result := FloatToStrF(nombre, ffFixed, 2, digits);
end;

function Tvoie.StrValeur(nombre: double): string;
var
   coeff: coefUnite;
   Negatif: boolean;
   tampon: string;
   i, N: integer;
begin
   if isNan(nombre) then
      Result := ''
   else begin
      if unite = '' then begin
         tampon := FloatToStrF(nombre, ffFixed, 4, 1);
         Result := tampon + ' ';
      end
      else begin
         negatif := nombre < 0;
         nombre := abs(nombre);
         coeff := nulle;
         while (nombre >= 999) and (coeff < infini) do begin
            nombre := nombre / 1000;
            Inc(coeff);
         end; { nombre<1000 ou infini }
         while (nombre < 0.999) and (coeff > zero) do begin
            nombre := nombre * 1000;
            Dec(coeff);
         end; { nombre>=1 ou zero }
         if coeff in [infini, zero] then
            tampon := '0 '
         else begin
            tampon := FloatToStrF(nombre, ffGeneral, 4, 0);
            i := 1;
            N := 0;
            while (N < 4) and (i < length(tampon)) do begin
               if charinset(tampon[i],['0'..'9']) then
                  Inc(N);
               Inc(i);
            end;
            // N=precision ou fin de chaine
            if N = 4 then
               tampon := copy(tampon, 1, i - 1);
            if negatif then
               tampon := '-' + tampon;
            tampon := tampon + ' ';
            if coeff <> nulle then
               tampon := tampon + symboleUnite[coeff];
         end;
         Result := tampon + unite;
      end;
   end;
end;


function Tvoie.AffVoltmetre: String;
begin
   if axedB then
      Result := nom + '=' + strValeurFixed(20 * log10(valeurCourante)) + 'dB'
   else
      Result := nom + '=' + strValeur(valeurCourante);
end;

procedure TvoieHP.load(ini: TmemInifile; nomIndex: String);
begin
   Nom  := Ini.ReadString(NomIndex, 'Nom', '');
   Unite := Ini.ReadString(NomIndex, 'Unite', '');
   Command := AnsiString(Ini.ReadString(NomIndex, 'Command', ''));
   CommandMX := AnsiString(Ini.ReadString(NomIndex, 'CommandMX', ''));
   Signif := Ini.ReadString(NomIndex, 'Signif', '');
   Positive := Ini.ReadBool(NomIndex, 'Positive', True);
   CalculIncertitude := TcalculIncertitude(
      Ini.ReadInteger(NomIndex, 'Incertitude', 0));
   Maxi := Ini.ReadFloat(NomIndex, 'Maxi', maxi);
   if positive then
      mini := 0
   else
      mini := -maxi;
end;


end.

